function [W, obj]= R_Elnet(X, Y, r1)

%% Problem
%  Elastic net
%  min_W  || W' X - Y||^2 + r1 * ||W||_2 + (1-r1) * ||W||_1
%  Input: (n: number of subjects, c: number of tasks, d: number of features)
%       - X: d x n
%       - Y: c x n
%       - r1: tuned parameters
%  Output:
%       - W: d x c
%       - obj: results of each iteration
%  Author: Jingwen Yan
%  Last update: 10/09/2012

%%
d = size(X,1);
c = size(Y,1);
XX = X * X';
XY = X * Y';
d1 = ones(d,c);

W = zeros(d,c);
for iter = 1:100
    
    for j = 1 : c
        D1 = diag(d1(:,j));
        Wj = (XX+r1*eye(d)+(1-r1)*D1)\XY(:,j);
        
        Wj_lasso = sqrt(sum(Wj.*Wj,2)+eps);
        d1(:,j) = 0.5./(Wj_lasso);
        
        W(:,j) = Wj;
    end
    
    obj(iter) = trace((W'*X-Y)'*(W'*X-Y)) + r1*sum(sum(abs(W)));
    
    if iter > 1 && abs(obj(iter) - obj(iter-1)) < 10e-3
        break;
    end
    
end;
