function [X, obj]= R_GL21(X, Y, class_idx, r1, r2)
%% 2-norm loss with group 21-norm regularization

%% Problem

% Problem
%
%  min_X  || W' X - Y||^2 + r1 * ||W||_21group + r2*||W||_21
%  Input: (n: number of subjects, c: number of tasks, d: number of features)
%       - X: d x n
%       - Y: c x n
%       - alpha: p x d, neighboring matrix
%       - r1, r2: tuned parameters
%  Output:
%       - W: d x c 
%       - obj: results of each iteration
%  Author: Jingwen Yan
%  Last update: 10/09/2012

d = size(X,1);
class_set = unique(class_idx);
class_num = length(class_set);

XX = X * X';
XY = X * Y';
d1 = ones(d,1);
d2 = ones(d,1);
Wi = zeros(d,1);

for iter = 1:100
    
    D1 = diag(d1);
    D2 = diag(d2);
    W = (XX+r1*D1+r2*D2)\XY;
    
    for c = 1:class_num
        idx = find(class_idx==class_set(c));
        Wc = W(idx,:);
        di = sqrt(sum(sum(Wc.*Wc))+eps);
        Wi(idx) = di;
        ob(c) = di;
    end;
    d1 = 0.5./Wi;
    
    Wi = sqrt(sum(W.*W,2)+eps);
    d2 = 0.5./(Wi);
    ob2 = sum(Wi);
    
    obj(iter) = trace((W'*X-Y)'*(W'*X-Y)) + r1*sum(ob) + r2*ob2;
    
    if iter > 1 && abs(obj(iter) - obj(iter-1)) < 10e-3
       break; 
    end
    
end;

