function [W, obj]= R_L21(X, Y, r1)

%% Problem
%
%  min_W  || W' X - Y||^2 + r1 * ||W||_21
%  Input: (n: number of subjects, c: number of tasks, d: number of features)
%       - X: d x n
%       - Y: c x n
%       - r1, r2: tuned parameters
%  Output:
%       - W: d x c 
%       - obj: results of each iteration
%  Author: Jingwen Yan
%  Last update: 10/09/2012

%%
[n, c] = size(X);
XX = X * X';
XY = X * Y';
d1 = ones(n,1);

for iter = 1:100

    D1 = diag(d1);
    W = (XX+r1*D1)\XY;
        
    Xi = sqrt(sum(W.*W,2)+eps);
    d1 = 0.5./(Xi);
    ob1 = sum(Xi);
    
    obj(iter) = trace((W'*X-Y)'*(W'*X-Y)) + r1*ob1;
    
    if iter > 1 && abs(obj(iter) - obj(iter-1)) < 10e-3
       break; 
    end
    
end;
