function [W, obj]= R_NGL21(X, Y, alpha, r1, r2)
%% 2-norm loss with RVoXM kernel

% Problem
%
%  min_W  || W' X - Y||^2 + r1 * ||a W||^2 + r2*||W||_21
%  Input: (n: number of subjects, c: number of tasks, d: number of features)
%       - X: d x n
%       - Y: c x n
%       - alpha: p x d, neighboring matrix
%       - r1, r2: tuned parameters
%  Output:
%       - W: d x c 
%       - obj: results of each iteration
%  Author: Jingwen Yan
%  Last update: 10/09/2012

%%

d = size(X,1);
D1 = alpha' * alpha;
XX = X * X';
XY = X * Y';
d2 = ones(d,1);

for iter = 1:100

    D2 = diag(d2);
    W = (XX+r1*D1+r2*D2)\XY;
    
    alphaW = alpha * W;
    ob = sum(sum(alphaW.*alphaW)+eps);
    
    Xi = sqrt(sum(W.*W,2)+eps);
    d2 = 0.5./(Xi);
    ob2 = sum(Xi);
    
    obj(iter) = trace((W'*X-Y)'*(W'*X-Y)) + r1*sum(ob) + r2*ob2;
    
    if iter > 1 && abs(obj(iter) - obj(iter-1)) < 10e-3
       break; 
    end
end;
